;===========================================================================================
; loadgif.asm  Nasm assembler
; Charger un fichier gif87a en mode plein cran              
; Programm by AsmGges France                                   http://www.chez.com/asmgges/
;===========================================================================================
; Compile:
; ..\..\bin\nasmw -f bin loadgif.asm -o loadgif.exe    
;===========================================================================================

%define ONLY_NASM 
%include "..\..\inc\nagoa+.inc"
%include "..\..\inc\defines.inc"
%include "ggesdx\ddrawdx.inc"

%define WSCREEN 640
%define HSCREEN 480

;Dclaration des procdures utilises par le programme
;-----------------------------------------------------
def CreateSurf
def FillSurface
def LoadGif87a
def LoadPalGif87a
def ZeroMemory
def FpsLimiter
def LibereSurf

;===========================================================================================
section .code

start:  

    invoke GetModuleHandle,NULL
    mov [wc+WNDCLASS.hInstance],eax

    call ZeroMemory,wc,WNDCLASS_size
    mov D[wc+WNDCLASS.lpfnWndProc],WindowProc
    mov D[wc+WNDCLASS.lpszClassName],ClassName

    invoke LoadCursor,NULL,IDC_ARROW
    mov [wc+WNDCLASS.hCursor],eax

    invoke RegisterClass,wc

    invoke GetSystemMetrics,SM_CXSCREEN
    mov [rectuser+RECT.right],eax    ;sauve le clipping actuel en x dans rectuser (WinMode)
    sub eax,WSCREEN
    shr eax,1
    mov [rect2+RECT.left],eax

    invoke GetSystemMetrics,SM_CYSCREEN
    mov [rectuser+RECT.bottom],eax   ;sauve le clipping actuel en y dans rectuser (WinMode)
    sub eax,HSCREEN
    shr eax,1
    mov [rect2+RECT.top],eax

    ;----------------------------------
    ; Cration fenetre pour plein cran
    ;----------------------------------
    invoke CreateWindowEx,0,ClassName,WinCaption,WS_POPUP+WS_VISIBLE,\
                               0,0,0,0,0,0,[wc+WNDCLASS.hInstance],0
    mov [hWin],eax

    invoke ShowWindow,[hWin],SW_SHOWDEFAULT
    invoke UpdateWindow,[hWin]

    ;-----------------------------
    ; Initialisation de DirectDraw
    ;-----------------------------
    invoke DirectDrawCreate,0,DDraw,0
    or eax,eax
    jnz near DdrawError

    ;--------------------
    ; Demande plein cran
    ;--------------------
    ddcall DD_SetCooperativeLevel,[DDraw],[hWin],DDSCL_EXCLUSIVE+DDSCL_FULLSCREEN
    or eax,eax
    jnz near DdrawError

    ;---------------------------------
    ; Passage en 640x480 pixels 8 bits
    ;---------------------------------
    ddcall DD_SetDisplayMode,[DDraw],WSCREEN,HSCREEN,8
    or eax,eax
    jnz near DdrawError

    ;----------------------------------------------------
    ; Cration d'une surface primaire avec un back-buffer
    ;----------------------------------------------------
    call ZeroMemory,ddsd,DDSURFACEDESC_size
    mov D[ddsd+DDSURFACEDESC.dwSize],DDSURFACEDESC_size
    mov D[ddsd+DDSURFACEDESC.dwFlags],DDSD_CAPS + DDSD_BACKBUFFERCOUNT
    mov D[ddsd+DDSURFACEDESC.ddsCaps],DDSCAPS_PRIMARYSURFACE+DDSCAPS_FLIP+DDSCAPS_COMPLEX
    mov D[ddsd+DDSURFACEDESC.dwBackBufferCount],1
    ddcall DD_CreateSurface,[DDraw],ddsd,Primary,0
    or eax,eax
    jnz near DdrawError

    call ZeroMemory,ddscaps,DDSCAPS_size
    mov D[ddscaps+DDSCAPS.dwCaps],DDSCAPS_BACKBUFFER
    ddcall DDS_GetAttachedSurface,[Primary],ddscaps,Back
    or eax,eax
    jnz near DdrawError

    ;----------------------------
    ; Cration d'une surface Fond
    ;----------------------------
    call CreateSurf,Fond,WSCREEN,HSCREEN
    or eax,eax
    jnz near DdrawError

    ;--------------------------
    ; Charger un fichier gif87a
    ;--------------------------
    call LoadGif87a,GifFile,[Fond]
    or eax,eax
    jnz near DataError

    ;---------------------------
    ; Charger une palette Gif87a
    ;---------------------------
    call LoadPalGif87a,GifFile
    or eax,eax
    jnz near DataError

    ;------------------------------------
    ; Palette Gif87a dans surface Primary
    ;------------------------------------
    ddcall DDS_SetPalette,[Primary],[GifPal]

    invoke timeBeginPeriod,1
    mov D[FpsMax],60

    jmp MenuLoop

Debut:


;===========================================================================================

GameLoop:

    ;----------------------
    ; Refresh: Back surface
    ;----------------------
    mov D[rect1+RECT.left],0
    mov D[rect1+RECT.top],0
    mov D[rect1+RECT.right],WSCREEN
    mov D[rect1+RECT.bottom],HSCREEN
    ddcall DDS_BltFast,[Back],0,0,[Fond],rect1,DDBLTFAST_NOCOLORKEY+DDBLTFAST_WAIT

    jmp MessageLoop

;===========================================================================================
MenuLoop:



;===========================================================================================

MessageLoop:

    invoke PeekMessage,msg,0,0,0,PM_NOREMOVE
    .if eax,ne,0
        invoke GetMessage,msg,0,0,0
        .if eax,e,0
            jmp ExitGame
        .endif
        invoke TranslateMessage,msg
        invoke DispatchMessage,msg
        jmp MessageLoop
    .endif

    .if B[bActive],e,0
        invoke WaitMessage
        jmp MessageLoop
    .endif

    ddcall DDS_IsLost,[Primary]
    .if eax,ne,0
        .if eax,e,DDERR_SURFACELOST
            ddcall DDS_Restore,[Primary]
        .else
            jmp ExitGame
        .endif
    .endif

    ;-------------------------------------------------
    ; Rgule la vitesse de rafraichissement de l'cran
    ;-------------------------------------------------
    call FpsLimiter,[FpsMax]

    ;-----
    ; Flip
    ;-----
    ddcall DDS_Flip,[Primary],0,DDFLIP_WAIT

    ;-----------------------
    ; Ecran de dessin et jeu
    ;-----------------------
    jmp GameLoop

;===========================================================================================

DdrawError:
    mov eax,MsgDdError
    jmp Error

DataError:
    mov eax,MsgDataError

Error:
    invoke MessageBox,[hWin],eax,WinCaption,0
    call LibereSurf
    invoke DestroyWindow,[hWin]
    invoke PostQuitMessage,1
    jmp MessageLoop

ExitGame:
    invoke ExitProcess,[msg+MSG.wParam]

;===========================================================================================
;===========================================================================================
proc WindowProc, hWnd, uMsg, wParam, lParam
    uses ebx,esi,edi

    .if D[@uMsg],e,WM_CLOSE
        call LibereSurf

    .elseif D[@uMsg],e,WM_ACTIVATE
        mov eax,[@wParam]
        mov [bActive],al

    .elseif D[@uMsg],e,WM_KEYDOWN
        .if D[@wParam],e,VK_ESCAPE
            invoke SendMessage,[@hWnd],WM_CLOSE,0,0
        .endif

    .else
        invoke DefWindowProc,[@hWnd],[@uMsg],[@wParam],[@lParam]
        return

    .endif

    mov eax,0

endproc
;===========================================================================================
; Libre les surfaces
;===========================================================================================
LibereSurf:

    invoke timeEndPeriod,1

    .if D[DDraw],ne,0
        .if D[Fond],ne,0
            ddcall DDS_Release,[Fond]
            mov D[Fond],0
        .endif
        .if D[lpClipper],ne,0
            ddcall DDC_Release,[lpClipper]
            mov D[lpClipper],0
        .endif
        .if D[Back],ne,0
            ddcall DDS_Release,[Back]
            mov D[Back],0
        .endif
        .if D[Primary],ne,0
            ddcall DDS_Release,[Primary]
            mov D[Primary],0
        .endif
        ddcall DD_Release,[DDraw]
        mov D[DDraw],0
    .endif
 
    invoke PostQuitMessage,0

    ret
;===========================================================================================

 %include "ggesdx\directxproc.asm"
 %include "ggesdx\generalproc.asm"

;===========================================================================================
section .data

 ClassName     db 'AsmGges Game',0
 WinCaption    db ' AsmGges Win32Asm Nasm DirectX',0
 BoxCaption    db ' AsmGges Win32Asm',0
 MsgDdError    db 'Direct Draw: failed',0
 MsgDataError  db 'Opening data file: failed',0
 GifFile       db 'res\nasm.gif',0

 align 4

 hdc           dd 0
 hWin          dd 0

 FullMode      dd 0
 HBarTitle     dd 0
 bActive       db 0
 align 4

 FpsMax        dd 0
 NextTimeEdx   dd 0
 NextTimeEax   dd 0
 PerfFreq      dq 0.0
 PerfCounter   dq 0.0

 DDraw         dd 0 
 lpClipper     dd 0 

 Primary       dd 0   
 Back          dd 0  
 Fond          dd 0

section .bss

 wc         resb WNDCLASS_size    
 msg        resb MSG_size         
 rect1      resb RECT_size         
 rect2      resb RECT_size        
 rectuser   resb RECT_size           

 ddsd       resb DDSURFACEDESC_size  
 ddcolorkey resb DDCOLORKEY_size    
 ddscaps    resb DDSCAPS_size       
 ddpf       resb DDPIXELFORMAT_size  
 ddbltfx    resb DDBLTFX_size       

 GifPal     resd 1
 NbrBytes   resd 1
 LBits      resb 1
 LTable     resd 7676     
 GifBuffer  resb 040000h  

;===========================================================================================
; Inclusion des fichiers .dll contenant les fonctions Windows utilises
;-------------------------------------------------------------------------------------------

LAST_BEGIN

__LIBS_   kernel32,'kernel32.DLL',\
          user32,  'user32.DLL',\
          gdi32,   'gdi32.DLL',\
          winmm,   'winmm.DLL',\
          ddraw,   'ddraw.DLL'

__IMPORT_ kernel32,\
          GetModuleHandle,'GetModuleHandleA',0,\
          CreateFile,'CreateFileA',0,\
          WriteFile,'WriteFile',0,\
          ReadFile,'ReadFile',0,\
          CloseHandle,'CloseHandle',0,\
          GetTickCount,'GetTickCount',0,\
          QueryPerformanceCounter,'QueryPerformanceCounter',0,\
          QueryPerformanceFrequency,'QueryPerformanceFrequency',0,\
          Sleep,'Sleep',0,\
          ExitProcess,'ExitProcess',0

__IMPORT_ user32,\
          RegisterClass,'RegisterClassA',0,\
          RegisterClassEx,'RegisterClassExA',0,\
          BeginPaint,'BeginPaint',0,\
          EndPaint,'EndPaint',0,\
          DialogBoxParam,'DialogBoxParamA',0,\
          EndDialog,'EndDialog',0,\
          SetTimer,'SetTimer',0,\
          KillTimer,'KillTimer',0,\
          LoadAccelerators,'LoadAcceleratorsA',0,\
          TranslateAccelerator,'TranslateAcceleratorA',0,\
          LoadBitmap,'LoadBitmapA',0,\
          LoadMenu,'LoadMenuA',0,\
          InvalidateRect,'InvalidateRect',0,\
          SendDlgItemMessage,'SendDlgItemMessageA',0,\
          CheckMenuItem,'CheckMenuItem',0,\
          CreateWindowEx,'CreateWindowExA',0,\
          ShowWindow,'ShowWindow',0,\
          UpdateWindow,'UpdateWindow',0,\
          DestroyWindow,'DestroyWindow',0,\
          DefWindowProc,'DefWindowProcA',0,\
          PeekMessage,'PeekMessageA',0,\
          TranslateMessage,'TranslateMessage',0,\
          DispatchMessage,'DispatchMessageA',0,\
          GetSystemMetrics,'GetSystemMetrics',0,\
          AdjustWindowRectEx,'AdjustWindowRectEx',0,\
          GetClientRect,'GetClientRect',0,\
          GetWindowRect,'GetWindowRect',0,\
          LoadCursor,'LoadCursorA',0,\
          LoadIcon,'LoadIconA',0,\
          ShowCursor,'ShowCursor',0,\
          ClipCursor,'ClipCursor',0,\
          DrawText,'DrawTextA',0,\
          MessageBox,'MessageBoxA',0,\
          WaitMessage,'WaitMessage',0,\
          GetMessage,'GetMessageA',0,\
          PostQuitMessage,'PostQuitMessage',0,\
          SendMessage,'SendMessageA',0,\
          LoadImage,'LoadImageA',0,\
          GetAsyncKeyState,'GetAsyncKeyState',0,\
          GetDC,'GetDC',0,\
          ReleaseDC,'ReleaseDC',0       

__IMPORT_ gdi32,\
          GetDeviceCaps,'GetDeviceCaps',0,\
          SetPixel,'SetPixel',0,\
          SetBkColor,'SetBkColor',0,\
          SetTextColor,'SetTextColor',0,\
          TextOut,'TextOutA',0,\
          GetPixel,'GetPixel',0,\
          GetStockObject,'GetStockObject',0,\
          GetObject,'GetObjectA',0,\
          CreateCompatibleDC,'CreateCompatibleDC',0,\
          CreateCompatibleBitmap,'CreateCompatibleBitmap',0,\
          SelectObject,'SelectObject',0,\
          DeleteObject,'DeleteObject',0,\
          DeleteDC,'DeleteDC',0,\
          BitBlt,'BitBlt',0 

__IMPORT_ winmm,\
          timeBeginPeriod,'timeBeginPeriod',0,\
          timeEndPeriod,'timeEndPeriod',0,\
          PlaySound,'PlaySoundA',0,\
          mciSendCommand,'mciSendCommandA',0

__IMPORT_ ddraw,\
          DirectDrawCreate,'DirectDrawCreate',0

LAST_END

;===========================================================================================
;///////////////////////////////////////////////////////////////////////////////////////////
;===========================================================================================
